(function() {
  'use strict';

  angular.module('journalism_workflow.v1.photographer.module', [
    'journalism_workflow.v1.photographer.controllers',
    'journalism_workflow.v1.photographer.directives'
  ]);

  angular.module('journalism_workflow.v1.photographer.controllers', []);
  angular.module('journalism_workflow.v1.photographer.directives', []);

})();
